package zero1hd.rhythmbullet.graphics.ui.pages;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Array;

import zero1hd.rhythmbullet.audio.SongInfo;
import zero1hd.rhythmbullet.audio.SongList;
import zero1hd.rhythmbullet.graphics.ui.components.MusicSelectable;

public class MusicSelectionPage extends Page {
	Preferences musicFileAnnotation;
	
	private SongList songList;
	private Array<MusicSelectable> selectables;
	private Table songTable;
	private ScrollPane scrollbar;
	private TextButton back;
	
	private FileHandle selectedMusic;
	private SongInfo selectedMusicInfo;
	
	private Skin skin;
	private AssetManager assets;
	
	private Image vertSplitBar;
	private Image horSplitBar;
	private Texture white;
	public MusicSelectionPage(Skin skin, SongList songList, AssetManager assetManager, Vector3 cameraTarget) {
		super("Select music", skin);
		setTextureBackground(assetManager.get("gradients.atlas", TextureAtlas.class).findRegion("red-round"));
		this.skin = skin;
		this.songList = songList;
		musicFileAnnotation = Gdx.app.getPreferences("music_file_annotation");
		this.assets = assetManager;
		songTable = new Table();
		songTable.defaults().spaceBottom(10f);
		scrollbar = new ScrollPane(songTable, skin);
		scrollbar.setSize(0.45f*getWidth(), getHeightBelowTitle());
		scrollbar.setFadeScrollBars(false);
		scrollbar.setOverscroll(false, false);
		addActor(scrollbar);
		selectables = new Array<>();
		back = new TextButton("Back", skin);
		back.setWidth(back.getWidth()+20f);
		back.setPosition(getWidth()-back.getWidth()-15f, getHeightBelowTitle());
		back.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				cameraTarget.x = 0.5f*Gdx.graphics.getWidth();
			}
		});
		addActor(back);
		
		back.toFront();
		refresh();
		
		Pixmap pixmap = new Pixmap(2, 2, Format.RGBA8888);
		pixmap.setColor(skin.getColor("default"));
		pixmap.fill();
		white = new Texture(pixmap);
		pixmap.dispose();
		
		vertSplitBar = new Image(white);
		vertSplitBar.setHeight(getHeight());
		vertSplitBar.setPosition(scrollbar.getWidth() + scrollbar.getX(), 0);
		addActor(vertSplitBar);
		
		horSplitBar = new Image(white);
		horSplitBar.setWidth(vertSplitBar.getX());
		horSplitBar.setPosition(0, getHeightBelowTitle());
		addActor(horSplitBar);
	}
	
	@Override
	public void act(float delta) {
		Gdx.gl.glLineWidth(2);
		super.act(delta);
	}
	
	public FileHandle getSelectedMusic() {
		return selectedMusic;
	}
	
	public SongInfo getSelectedMusicInfo() {
		return selectedMusicInfo;
	}
	
	public void refresh() {	
		songTable.clear();
		selectables.clear();
		for (int i = 0; i < selectables.size; i++) {
			selectables.get(i).dispose();
		}
		
		Gdx.app.debug("MusicSelectionPage", "Refreshing...");
		for (int i = 0; i < songList.getAmountOfSongs(); i++) {
			
			MusicSelectable selectable = new MusicSelectable(songList.getSongFileHandleFromIndex(i), musicFileAnnotation, skin, assets.get("defaultCover.png", Texture.class));
			selectables.add(selectable);
			
			songTable.add(selectable).expandX().fillX().height((getHeight() >= 2160 ? 300f : 200f));
			songTable.row();
		}
		
		ExecutorService exec = Executors.newSingleThreadExecutor();
		exec.submit(() -> {
			for (int i = 0; i < selectables.size; i++) {
				MusicSelectable info = selectables.get(i);
				info.getAudioInfo().loadInfo();
				info.getAudioInfo().doesContainsInformation();
				Gdx.app.postRunnable(() -> {
					info.updateInfo();
				});
			}
			Gdx.app.debug("MusicSelectionPage", "Refresh complete. " + selectables.size + " songs loaded.");
		});
	}
	
	@Override
	public void dispose() {
		white.dispose();
		super.dispose();
	}
}
